/*
	Message renumbering utility for XFBB 7.01 or newer 
	Copyright (c) 2001 by Stewart Wilkinson (G0LGS) 
*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <fcntl.h>
#include <stdarg.h>
#include <string.h> 
#include <ctype.h>
#include <getopt.h>
#include <errno.h>
#include <unistd.h>
#include <sys/signal.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "utils.h"

extern char *optarg;
extern int optind;
extern int opterr;
extern int getopt(int argc, char * const *argv, const char *optstring);

char data_path[128];
char msg_path[128];
char bin_path[128];
char dirmes_sys[128];
char dirmes_new[128];
char dirmes_old[128];
char inf_sys[128];
char inf_new[128];
char inf_old[128];
char wf_sys[128];
char wf_new[128];
char wf_old[128];

long base;
int  lng = 0;
int  verbose = 0;

#define _read read
#define _write write
#define O_BINARY 0
#define __a2__ __attribute__ ((packed, aligned(2)))

#define NBBBS 80
#define NBMASK NBBBS / 8
#define	LGFREE	15

#define F_EXC	0x0001
#define F_LOC	0x0002
#define F_EXP	0x0004
#define F_SYS	0x0008
#define F_BBS	0x0010
#define F_PAG	0x0020
#define F_GST	0x0040
#define F_MOD	0x0080
#define F_PRV	0x0100
#define F_UNP	0x0200
#define F_NEW	0x0400
#define F_PMS	0x0800
 
#define EXC(flag)	(flag & F_EXC)
#define LOC(flag)	(flag & F_LOC)
#define EXP(flag)	(flag & F_EXP)
#define SYS(flag)	(flag & F_SYS)
#define BBS(flag)	(flag & F_BBS)
#define PAG(flag)	(flag & F_PAG)
#define GST(flag)	(flag & F_GST)
#define MOD(flag)	(flag & F_MOD)
#define PRV(flag)	(flag & F_PRV)
#define UNP(flag)	(flag & F_UNP)
#define NEW(flag)	(flag & F_NEW)
#define PMS(flag)	(flag & F_PMS)

typedef struct
{				/* = 194 bytes */
	char type;
	char status;
	long numero __a2__;
	long taille __a2__;
	long date __a2__;
	char bbsf[7];
	char bbsv[41];
	char exped[7];
	char desti[7];
	char bid[13];
	char titre[61];
	char bin;
	char free[9];
	unsigned short nblu;
	long theme __a2__;
	long datesd __a2__;
	long datech __a2__;
	char fbbs[NBMASK];
	char forw[NBMASK];
}
bullist;

typedef struct typindic
{
	char call[7];
	char num;
}
indicat;

typedef struct
{
	indicat indic;			/* 8  Callsign */
	indicat relai[8];		/* 64 Digis path */
	long	lastmes __a2__;		/* 4  Last L number */
	long	nbcon   __a2__;		/* 4  Number of connexions */
	long	hcon    __a2__;		/* 4  Last connexion date */
	long	lastyap __a2__;		/* 4  Last YN date */
	unsigned short	flags;		/* 2  Flags */
	unsigned short	on_base;	/* 2  ON Base number */
	unsigned char	nbl;		/* 1  Lines paging */
	unsigned char	lang;		/* 1  Language */
	long	newbanner __a2__;	/* 4  Last Banner date */
	unsigned short 	download  ;	/* 2  download size (KB) = 100 */
	char	free[20];		/* 20 Reserved */
	char	theme;			/* 1  Current topic */
	char	nom[18];		/* 18 1st Name */
	char	prenom[13];		/* 13 Christian name */
	char	adres[61];		/* 61 Address */
	char	ville[31];		/* 31 City */
	char	teld[13];		/* 13 home phone */
	char	telp[13];		/* 13 modem phone */
	char	home[41];		/* 41 home BBS */
	char	qra[7];			/* 7  Qth Locator */
	char	priv[13];		/* 13 PRIV directory */
	char	filtre[7];		/* 7  LC choice filter */
	char	pass[13];		/* 13 Password */
	char	zip[9];			/* 9  Zipcode */
					/* Total : 360 bytes */
}
info;

typedef struct
{
	char	mode;			/* 1  Type of Message */
	char	fbid[13];		/* 13 BID of Message */
	long	msg_number __a2__;	/* 4  Number of Message */
}
bidfwd;

#define FBBCONF "/etc/ax25/fbb.conf"

int ReadInit()
{
	int len, i;
	int end = 0;
	char buffer[256];
	char *scan;
	char *ptr;
	char key[20];
	char pathname[256];
	FILE *init_ptr;

	if( (ptr = getenv("FBBCONF")) == NULL ){
		strcpy( pathname, FBBCONF);
	}else{
		strcpy( pathname, ptr);
	}

	if( access( pathname, R_OK ) != 0 ){
		fprintf( stderr, "Cannot find %s\n", pathname );
		return 1;
	}

	if ((init_ptr = fopen (pathname, "r")) != NULL) {

		if( verbose ){
			fprintf( stdout, "Reading %s.\n", pathname );
		}

		while( !end && fgets(buffer, sizeof(buffer), init_ptr) ){

			scan = buffer;

			/* Eat Spaces */
			while (isspace(*scan))
				++scan;

			/* Comments & Blanks ignored */
			if (*scan == '#' || *scan == '\0' )
				continue;

			/* remove leading \n */
			len = strlen(scan);
			if( len && (scan[len-1] == '\n'))
				scan[len-1] = '\0';

			/* keyword */
			i=0;
			ptr = key;
			while( isgraph(*scan)) {
				if( i++ < 4 ) {
					*ptr++ = *scan;
				}
				++scan;
				if(*scan == '=')
					break;         
			}
			*ptr = '\0';

			/* find the '=' */
			scan = strchr(scan, '=');

			/* copy value */
			if(scan) {
				++scan;
				while( isspace(*scan))
					++scan;

				/* if 1st char is '"' then string */
				if( *scan == '"' ){
					++scan;
					ptr = strrchr(scan,'"');
					if(ptr)
						*ptr = '\0';
				}

				if( strcmp( key, "data" ) == 0 ) {
					strcpy( data_path, scan );

				}else if( strcmp( key, "mess" ) == 0 ) {
					strcpy( msg_path, scan );

				}else if( strcmp( key, "comp" ) == 0 ) {
					strcpy( bin_path, scan );

				}

			}
		}

		fclose(init_ptr);
  
	}else{
		fprintf( stderr, "Unable to open '%s'\n", pathname );
		return 1;
	}

	return 0;
}

void do_head(char *ap)
{
	fprintf( stderr, "\n%s V" Version " " Vdate " for XFBB V7.01+ (c) by Stewart Wilkinson (G0LGS)\n", ap );
}

void do_usage(char *cmd)
{
	do_head( cmd );

	if( (strcmp( cmd, "fbbrenum" ) == 0) || (strcmp( cmd, "renum") ==0) ) {

		fprintf( stderr, "\n\tUsage: %s start_number\n\n\007", cmd );

	}else {

		fprintf( stderr, "\n\tUsage: %s\n\n\007", cmd );
	}

}


int Is_fbb(void) {

	int pid;

	pid = WEXITSTATUS(system("pidof xfbb >/dev/null 2>&1"));

	if( (pid != -1) && (pid != 0) ){
		pid = WEXITSTATUS(system("pidof xfbbd >/dev/null 2>&1"));
	}

	return(pid);
}

char *strflags (info * frec)
{
#define NB_FLAG 11
	int i;
	static char flag[NB_FLAG + 1];
 
	for (i = 0; i < NB_FLAG; i++)
		flag[i] = '.';
	flag[NB_FLAG] = '\0';
 
	if (NEW (frec->flags))
		flag[10] = 'I';
	if (UNP (frec->flags))
		flag[9] = 'U';
	if (MOD (frec->flags))
		flag[8] = 'M';
	if (EXC (frec->flags))
		flag[7] = 'E';
	if (PMS (frec->flags))
		flag[6] = 'F';
	if (LOC (frec->flags))
		flag[5] = 'L';
	if (EXP (frec->flags))
		flag[4] = 'X';
	if (SYS (frec->flags))
		flag[3] = 'S';
	if (BBS (frec->flags))
		flag[2] = 'B';
	if (PAG (frec->flags))
		flag[1] = 'P';
	if (PRV (frec->flags))
		flag[0] = 'R';
 
	return (flag);
}

int do_usrlist(char *file)
{
	info userrec;
	long count = 0L;
	long good = 0L;
	FILE *ptr;

	if( access( file, R_OK ) != 0 ){
		fprintf( stderr, "unable to find %s\n", file );
		return 1;
	}

	/* Open */
 	ptr = fopen( file, "rb");
	if ( ptr == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", file );
		return 1; 
	}

	fprintf( stdout, "\nCallsign  LastConnect #Cnct RPBSXLFEMUI Lg Name      Password     PRIV-Dir.\n" );

	while( fread(&userrec, sizeof(info), 1, ptr) && ! feof( ptr ) ){

		count++;
		if( *userrec.indic.call != '\0' ){

			good++;

			fprintf( stdout, "%-6s-%-2d ", userrec.indic.call, userrec.indic.num );
			fprintf( stdout, "%s ", strdt(userrec.hcon) );
			fprintf( stdout, "%5ld ", userrec.nbcon );
			fprintf( stdout, "%s ", strflags(&userrec) );
			fprintf( stdout, "%-12s ", strlen(userrec.prenom) ? userrec.prenom : "?" );
			fprintf( stdout, "%-12s ", userrec.pass );
			fprintf( stdout, "%-11s\n", userrec.priv );

			if( lng ) {
				fprintf( stdout, "HomeBBS: %-40s ", strlen(userrec.home) ? userrec.home : "?" );
				fprintf( stdout, "Zip/Locator : %s\n", strlen(userrec.zip) ? userrec.zip : "?" );
			}


		}
	}


	fclose( ptr );

	fprintf( stdout, "\n%ld Records\n", count );
	fprintf( stdout, "%ld Known Users\n", good );

	return 0;

}

int do_msglist(char *file)
{

	bullist	bullrec;
	long count = 0L;
	FILE *ptr;

	if( access( file, R_OK ) != 0 ){
		fprintf( stderr, "unable to find %s\n", file );
		return 1;
	}

	/* Open */
 	ptr = fopen( file, "rb");
	if ( ptr == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", file );
		return 1; 
	}

	/* Skip the First Record */
	fseek( ptr, sizeof(bullist), SEEK_SET );

	fprintf( stdout, "\n  Msg# TS   Size To     @ BBS  From   Date   Time MID/BID       Subject\n" );
	fprintf( stdout, "=======================================================================\n" );

	while( fread(&bullrec, sizeof(bullist), 1, ptr)  && ! feof( ptr ) ){

		count++;

		if( strstr(bullrec.bbsv, ".") ) {
			*strstr(bullrec.bbsv, ".") = '\0';
		}

		fprintf( stdout, "%6ld ", bullrec.numero );
		fprintf( stdout, "%c", bullrec.type );
		fprintf( stdout, "%c ", bullrec.status );
		fprintf( stdout, "%6ld ", bullrec.taille );
		fprintf( stdout, "%-6s ", bullrec.desti );
		fprintf( stdout, "%-6.6s ", bullrec.bbsv );
		fprintf( stdout, "%-6s ", bullrec.exped );
		fprintf( stdout, "%s ", strdt(bullrec.date) );
		fprintf( stdout, "%-13s ", bullrec.bid );
		fprintf( stdout, "%-61s", bullrec.titre );
		fprintf( stdout, "\n" );
	}


	fprintf( stdout, "\n%ld Total Messages\n", count );

	fclose( ptr );

	return 0;
}

int do_ren(void)
{
	bullist	bullrec;
	info	userrec;
	bidfwd	bidrec;
	long count = 0L;
	long missing = 0L;
	long min = LONG_MAX;
	long max = LONG_MIN;
	long range;
	long diff;
	char oldmsg[128];
	char newmsg[128];
	int unit;
	FILE *dirsys, *dirnew;
	FILE *infsys, *infnew;
	FILE *wfsys, *wfnew;
	long lastbid __a2__;
	
	if( ! copy_ ( dirmes_sys, dirmes_old ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", dirmes_sys, dirmes_old );
		return 1; 
	}

	/* Open dirmes.sys */
 	dirsys = fopen( dirmes_sys, "rb");
	if (dirsys == NULL) {
		fprintf( stderr, "Unable to open '%s'\n", dirmes_sys );
		return 1; 
	}

	/* Skip the First Record during initial checks - as it seems to be a duplicate of last message number */
	fseek( dirsys, sizeof(bullist), SEEK_SET );

	if( ! verbose ){
		fprintf( stdout, "Checking " );
		fflush( stdout );
	}

	count = 0;

	/* Find range of existing message numbers & check status of message files */
	while( fread(&bullrec, sizeof(bullist), 1, dirsys)  && ! feof( dirsys ) ){

		count++;
		if( (! (count % 50) ) && (! verbose) ){
			fprintf( stdout, ".");
			fflush( stdout );
		}

		if( verbose ){
			fprintf( stdout, "Checking Message %ld.", bullrec.numero );
		}

		if( bullrec.numero < min )
			min = bullrec.numero;

		if( bullrec.numero > max )
			max = bullrec.numero;

		unit = (unsigned int) bullrec.numero % 10;

		/* Message Location */
		sprintf( oldmsg, "%s/mail%d/m_%06ld.mes", msg_path, unit, bullrec.numero );

		if( access( oldmsg, R_OK ) != 0 ){

			missing++;

			if( verbose ){
				fprintf( stdout, "- Missing.\n" );
			}

		}else if( verbose ){
			fprintf( stdout, " - Ok.\n" );
		}



	}

	if( ! verbose ){
		fprintf( stdout, "\n" );
	}

	fclose( dirsys );

	if( missing ) {
		fprintf( stdout, "\n%ld Missing message files\n", missing );
	}

	/* The Adjust the base number so we don't have to move files into different mail folders */
	while( (base % 10) != (min % 10) ) {
		base++;
	}

	range = max - min;
	diff = min - base;

	/* Ensure messages will fit in new range */
	if( ((base >= min) && (base <= max)) || ( (base < min) && ((base+range) >= min) ) ){
		fprintf( stderr, "Sorry: Cannot renumber starting at %ld, new & old number ranges overlap\n", base );
		fprintf( stderr, "\n" );
		return 1;
	}

	if( ((base+range) >= 999999 ) ){
		fprintf( stderr, "Sorry: new starting number (%ld) is too large\n", base );
		fprintf( stderr, "\n" );
		return 1;
	}

	/* Open dirmes.sys */
 	dirsys = fopen( dirmes_sys, "rb");
	if (dirsys == NULL) {
		fprintf( stderr, "Unable to open '%s'\n", dirmes_sys );
		return 1; 
	}

	/* Open dirmes.new */
 	dirnew = fopen( dirmes_new, "wb");
	if (dirnew == NULL) {
		fprintf( stderr, "Unable to create '%s'\n", dirmes_new );
		return 1; 
	}

	fprintf( stdout, "\n Lowest Message number will be %ld\n", base );
	fprintf( stdout, "Largest Message number will be %ld\n", base + range );

	fprintf( stdout, "\nRenumbering (1) " );
	if( verbose ){
		fprintf( stdout, "\n" );
	}
	fflush( stdout );

	/* Renumber Existing dirmes.sys to dirmes.new */
	count = 0;
	while( fread(&bullrec, sizeof(bullist), 1, dirsys)  && ! feof( dirsys ) ){

		count++;
		if( (! (count % 50)) && ( ! verbose) ){
			fprintf( stdout, ".");
			fflush( stdout );
		}

		if( verbose ){
			fprintf( stdout, "%ld -> ", bullrec.numero );
		}

		bullrec.numero -= diff; 

		if( verbose ){
			fprintf( stdout, "%ld\n", bullrec.numero );
		}

		if( ( fwrite( &bullrec, sizeof(bullist), 1, dirnew ) == 0 ) ) {
			fprintf( stderr, "Error: writing to %s\n", dirmes_new );
			fprintf( stderr, "Cannot Re-number\n" );
			fclose( dirsys );
			fclose( dirnew );
			return 1;
		}

	}

	/* Finished creating dirmes.new so close it */
	fclose( dirnew );

	/* Ignore SIGINT (CTRL-C) - as it will mess up XFBB if interrupted */
	signal( SIGINT, SIG_IGN );

	/* Start at 2nd record of dirmes.sys */
	fseek( dirsys, sizeof(bullist), SEEK_SET );

	fprintf( stdout, "\nRenumbering (2) " );
	if( verbose ){
		fprintf( stdout, "\n" );
	}
	fflush( stdout );


	/* Rename Message files using original dirmes.sys */
	count = 0;
	while( fread(&bullrec, sizeof(bullist), 1, dirsys)  && ! feof( dirsys ) ){

		count++;
		if( (! (count % 50)) && (! verbose) ){
			fprintf( stdout, ".");
			fflush( stdout );
		}

		/* Old Message Location */
		unit = (unsigned int) bullrec.numero % 10;
		sprintf( oldmsg, "%s/mail%d/m_%06ld.mes", msg_path, unit, bullrec.numero );

		if( verbose ){
			fprintf( stdout, "%ld -> ", bullrec.numero );
		}

		bullrec.numero -= diff; 

		if( verbose ){
			fprintf( stdout, "%ld\n", bullrec.numero );
		}

		/* New Message Location */
		unit = (unsigned int) bullrec.numero % 10;
		sprintf( newmsg, "%s/mail%d/m_%06ld.mes", msg_path, unit, bullrec.numero );

		if( rename( oldmsg, newmsg ) != 0 ){
			fprintf( stderr, "Error: unable to rename %s to %s\n", oldmsg, newmsg );
		}

		/* Remove any Compressed Message file */
		sprintf( oldmsg, "%s/mail%d/m_%06ld.mes", bin_path, unit, bullrec.numero );

		if( access( oldmsg, W_OK ) == 0 ) {
			if( unlink( oldmsg ) != 0 ) {

				fprintf( stderr, "Error: unable to remove %s.\n", oldmsg );

			}else if( verbose ){
				fprintf( stdout, "Removed %s.\n", oldmsg );
			}

		}
	}

	if( ! verbose ) {
		fprintf( stdout, "\n" );	
	}

	/* close original dirmes.sys */
	fclose( dirsys );

	/* replace original dirmes.sys with copy of dirmes.new */
	if( ! copy_ ( dirmes_new, dirmes_sys ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", dirmes_new, dirmes_sys );
		fprintf( stderr, "Please copy or rename the file manually before starting XFBB\n" );
	}


	/* Now deal with inf.sys */
	fprintf( stdout, "\nAdjusting user settings " );
	fflush( stdout );

	if( ! copy_ ( inf_sys, inf_old ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", inf_sys, inf_old );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Open INF.SYS */
 	infsys = fopen( inf_sys, "rb");
	if ( infsys == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", inf_sys );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Open INF.NEW */
 	infnew = fopen( inf_new, "wb");
	if ( infnew == NULL ) {
		fprintf( stderr, "Unable to create '%s'\n", inf_new );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	count=0;
	while( fread(&userrec, sizeof(info), 1, infsys) && ! feof( infsys ) ){

		count++;
		if( ! (count % 10) ){
			fprintf( stdout, ".");
			fflush( stdout );
		}

		/* Ignore & filter out entries with null callsign field */
		if( *userrec.indic.call != '\0' ){

			/* Adjust last listed number */
			if( (userrec.lastmes >= min) && (userrec.lastmes <= max) ) {
				userrec.lastmes -= diff;
			}else{
				userrec.lastmes = 1;
			}

			/* Adjust Message Base Number */
			if( userrec.on_base > 0 ) {
				userrec.on_base -= (diff/1000);
			}else{
				userrec.on_base = 0;
			}

			/* Write the adjusted user record */
			if( ( fwrite( &userrec, sizeof(info), 1, infnew ) == 0 ) ) {
				fprintf( stderr, "Error: writing to %s\n", inf_new );
				fclose( infsys );
				fclose( infnew );

				/* Enable CTRL-C */
				signal( SIGINT, SIG_DFL );

				return 1;
			}
		}
	}

	fprintf( stdout, "\n" );	

	fclose( infsys );
	fclose( infnew );

	/* replace original inf.sys with copy of inf.new */
	if( ! copy_ ( inf_new, inf_sys ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", inf_new, inf_sys );
		fprintf( stderr, "Please copy the file manually before starting XFBB\n" );
	}


	/* Now deal with MID/BID (wfbid.sys) */
	fprintf( stdout, "\nAdjusting MID/BID History " );
	fflush( stdout );

	if( ! copy_ ( wf_sys, wf_old ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", wf_sys, wf_old );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Open wfbid.sys */
 	wfsys = fopen( wf_sys, "rb");
	if ( wfsys == NULL ) {
		fprintf( stderr, "Unable to open '%s'\n", wf_sys );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Open wfbid.new */
 	wfnew = fopen( wf_new, "wb");
	if ( wfnew == NULL ) {
		fprintf( stderr, "Unable to create '%s'\n", wf_new );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Read 1st record (Pointer) */
	fread(&bidrec, sizeof(bidfwd), 1, wfsys);

	/* last used record */
	lastbid = bidrec.msg_number;

	/* Write the record */
	if( ( fwrite( &bidrec, sizeof(bidfwd), 1, wfnew ) == 0 ) ) {
		fprintf( stderr, "Error: writing to %s\n", wf_new );
		fclose( wfsys );
		fclose( wfnew );

		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1;
	}
	
	count=0;
	while( fread(&bidrec, sizeof(bidfwd), 1, wfsys) && ! feof( wfsys ) ){

		count++;
		if( ! (count % 10) ){
			fprintf( stdout, ".");
			fflush( stdout );
		}

		/* Ignore & filter out entries with null Type or MID/BID fields */
		if( (bidrec.mode != '\0') && (*bidrec.fbid != '\0') ){

			/* Adjust Message number */
			if( (bidrec.msg_number >= min) && (bidrec.msg_number <= max) ) {
				bidrec.msg_number -= diff;
			}else{
				bidrec.msg_number = 0;
			}

			/* Write the adjusted MID/BID record */
			if( ( fwrite( &bidrec, sizeof(bidfwd), 1, wfnew ) == 0 ) ) {
				fprintf( stderr, "Error: writing to %s\n", wf_new );
				fclose( wfsys );
				fclose( wfnew );

				/* Enable CTRL-C */
				signal( SIGINT, SIG_DFL );
				return 1;
			}

		}else{
			/* Adjust Last Record Pointer to compensate for removing Bad entry from list */
			if( count <= lastbid ) {
				lastbid--;
			}
		}
	}

	fprintf( stdout, "\n" );	

	fclose( wfsys );
	fclose( wfnew );


	/* re-open wfbid.new */
 	wfnew = fopen( wf_new, "ab");
	if ( wfnew == NULL ) {
		fprintf( stderr, "Unable to create '%s'\n", wf_new );
		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1; 
	}

	/* Read 1st Record */
	fread( &bidrec, sizeof(bidfwd), 1, wfnew );
	
	/* adjust Pointer */
	bidrec.msg_number = lastbid;
	
	/* Move to begining of file */
	fseek( wfnew, 0, SEEK_SET );

	/* Write the Modified Pointer record */
	if( ( fwrite( &bidrec, sizeof(bidfwd), 1, wfnew ) == 0 ) ) {
		fprintf( stderr, "Error: writing to %s\n", wf_new );
		fclose( wfsys );

		/* Enable CTRL-C */
		signal( SIGINT, SIG_DFL );
		return 1;
	}
	
	fclose( wfnew );


	/* replace original wfbid.sys with copy of wfbid.new */
	if( ! copy_ ( wf_new, wf_sys ) ){
		fprintf( stderr, "ERROR: Unable to copy '%s' to '%s'\n", wf_new, wf_sys );
		fprintf( stderr, "Please copy the file manually before starting XFBB\n" );
	}


	/* Enable CTRL-C */
	signal( SIGINT, SIG_DFL );

	fprintf( stdout, "Finished.\n" );	
	return 0;
}

int main(int argc, char *argv[])
{
	char Cmd[128];
	char *ap;
	int ch;
	int option_index = 0;
	static struct option long_options[] =
	{
	{ "help", 0, 0, 'h' },
	{ "long", 0, 0, 'l' },
	{ "short", 0, 0, 's' },
	{ "version", 0, 0, 'v' },
	{ "verbose", 0, 0, 0 },
	{ 0, 0, 0, 0 }
	};

	char arg[80];
	char *p;
	int err;

	int fbbpid;

	strcpy( Cmd, argv[0] );
	if( (ap = rindex( Cmd, '/') ) != NULL ) {
		ap++;
	}else{
		ap=Cmd;
	}

	while( (ch = getopt_long( argc, argv, "?hHvVlLsS", long_options, &option_index ) ) != (-1) ) {

		switch(ch){

			case 0:
				if( strcmp(long_options[option_index].name, "verbose" ) == 0 ){
					verbose=1;
				}
				break;

			case '?':
			case 'h':
			case 'H':
				do_usage(ap);
				return 1;

			case 'v':
			case 'V':
				do_head(ap);
				return 1;

			case 'l':
			case 'L':
				lng=1;
				break;

			case 's':
			case 'S':
				lng=0;
				break;

			default:
				return 1;
		}
	}

	if( ReadInit() != 0 ){
		do_head(ap);
		fprintf( stderr, "Error Processing Config file.\n" );
		exit(1);
        }

	sprintf( dirmes_sys, "%s/dirmes.sys", data_path );
	sprintf( dirmes_new, "%s/dirmes.new", data_path );
	sprintf( dirmes_old, "%s/dirmes.old", data_path );

	sprintf( inf_sys, "%s/inf.sys", data_path );
	sprintf( inf_new, "%s/inf.new", data_path );
	sprintf( inf_old, "%s/inf.old", data_path );

	sprintf( wf_sys, "%s/wfbid.sys", data_path );
	sprintf( wf_new, "%s/wfbid.new", data_path );
	sprintf( wf_old, "%s/wfbid.old", data_path );

	if( access( dirmes_sys, W_OK ) != 0 ){
		fprintf( stderr, "Cannot find '%s'\n", dirmes_sys );
		exit(1); 
	}

	if( access( inf_sys, W_OK) != 0 ){
		fprintf( stderr, "Cannot find '%s'\n", inf_sys );
		exit(1); 
	}

	if( (strcmp( ap, "fbbrenum" ) ==0 ) || (strcmp( ap, "renum" ) ==0 ) ) {

		/* Check the Number Arguments */
		if ( (argc - optind) != 1){
			do_usage(ap);
			return 1;
		}

		/* Convert 1st argument to number */
		strcpy( arg, argv[optind++] );
		base = strtol( arg, &p, 0 );

		err = errno;
		if( err ){
			do_usage(ap);
			fprintf( stderr, "\n'%s' is not a valid number\n", arg );
			return 1;
		}

		if( p == arg ){
			do_usage(ap);
			fprintf( stderr, "\n'%s' is not a valid number\n", arg );
			return 1;
		}

		if( (base < 100) || ( base > 999000 ) ){
			do_head( ap );
			fprintf( stderr, "Start number is out of range (100 - 999,000)\n" );
			return 1;
		}

		do_head( ap );

		if( verbose ){
			fprintf( stdout, "Checking if XFBB is active.\n" );
		}

		fbbpid = Is_fbb();

		if( fbbpid == 127 ) {
			fprintf(stderr, "\007 Error: Unable to determine XFBB is currently active.\n" );
			return 2;
		}

		if( fbbpid == 0 ) {
			fprintf(stderr, "\007 Error: XFBB is currently active - Cannot re-number messages at this time.\n" );
			return 2;
		}

		err = do_ren();

	}else if( strcmp(ap, "listmsg" ) == 0 ) {

		do_head( ap );
		do_msglist(dirmes_sys);
		err = 0;

	}else if( strcmp(ap, "listusr") == 0 ) {

		do_head( ap );
		do_usrlist(inf_sys);
		err = 0;

	}else{
		fprintf( stderr, "name %s is unknown\n\007", ap );
		err = 255;
	}

	exit(err);
}

